;---------------------------------------------------------------
; This code is based on the video driver for FTS/Forth. Samuel
; Falvo II has kindly granted permission for me to redistribute
; and modify it without restrictions.  Thank you Sam!
;---------------------------------------------------------------

;****** uMoveDown *********************************************
;
; NAME
;       uMoveDown
;
; SYNOPSIS
;       mov     eax,srcAddress
;       mov     ebx,dstAddress
;       mov     ecx,nbrOfBytes
;       call    uMoveDown
;
; FUNCTION
;       Moves ECX bytes from EBX to EAX.  This move is assumed
;       correct if and only if EBX <= EAX.  Otherwise, memory 
;       is filled with a repeating pattern.
;
; INPUTS
;       EAX     source address
;       EBX     destination address
;       ECX     number of bytes to move
;
; RESULT
;
; BUGS
;
; SEE ALSO
;       uMoveUp, uMove
;
;**************************************************************
uMoveDown:
        push    eax
        push    ebx
        push    ecx
        push    edx

        cmp     ecx,4
        jb      .no32bits

.more32bits:
        mov     edx,[eax]
        add     eax,4
        mov     [ebx],edx
        add     ebx,4
        sub     ecx,4
        cmp     ecx,4
        jae     .more32bits

.no32bits:
        cmp     ecx,2
        jb      .no16bits

.more16bits:
        mov     dx,[eax]
        add     eax,2
        mov     [ebx],dx
        add     ebx,2
        sub     ecx,2
        cmp     ecx,2
        jae     .more16bits

.no16bits:
        or      ecx,ecx
        jz      .done

.morebytes:
        mov     dl,[eax]
        inc     eax
        mov     [ebx],dl
        inc     ebx
        dec     ecx
        or      eax,eax
        jnz     .morebytes

.done:
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret

